import React, { useState, useEffect, useRef, ReactNode } from 'react';
import styles from './dropdowns.module.scss';

type DropdownContainerProps = {
  children: ReactNode;
  className?: string;
};

export const DropdownContainer = React.forwardRef<HTMLDivElement, DropdownContainerProps>(
  ({ children, className }, ref: React.ForwardedRef<HTMLDivElement>) => {
    return (
      <div className={`border-1 border-secondary-800  relative inline-block text-left ${className && className}`} ref={ref}>
        {children}
      </div>
    );
  }
);

type DropdownButtonProps = {
  title: string | ReactNode;
  onClick: (e: React.MouseEvent<HTMLButtonElement>) => void;
  size?: 'xs' | 'sm' | 'md' | 'xl';
  disabled?: boolean;
};

export function DropdownButton({ title, onClick, size, disabled }: DropdownButtonProps) {
  return (
    <>
      <button
        className="inline-flex w-full justify-between items-center gap-x-1.5 rounded bg-light px-3 py-2 text-secondary-900 shadow-sm ring-1 ring-inset ring-secondary-300 hover:bg-secondary-50 disabled:bg-secondary-100 disabled:cursor-not-allowed disabled:text-secondary-400"
        onClick={onClick}
        disabled={disabled}
      >
        <span className={`text-${size}`}>{title}</span>
        <svg className="-mr-1 h-5 w-5 text-secondary-400" viewBox="0 0 20 20" fill="currentColor" aria-hidden="true">
          <path
            fillRule="evenodd"
            d="M5.23 7.21a.75.75 0 011.06.02L10 11.168l3.71-3.938a.75.75 0 111.08 1.04l-4.25 4.5a.75.75 0 01-1.08 0l-4.25-4.5a.75.75 0 01.02-1.06z"
            clipRule="evenodd"
          />
        </svg>
      </button>
    </>
  );
}

type DropdownItemContainerProps = {
  align: string;
  className?: string;
  children: ReactNode;
};

export function DropdownItemContainer({ align, className, children }: DropdownItemContainerProps) {
  return (
    <div
      className={`${styles['dropdown-container']} ${align} ${className} bg-light`}
      role="menu"
      aria-orientation="vertical"
      aria-labelledby="menu-button"
      tabIndex={-1}
    >
      <ul role="none">{children}</ul>
    </div>
  );
}

type DropdownItemProps = {
  value: string;
  disabled?: boolean;
  className?: string;
  onClick?: (value: string) => void;
  submenu?: React.ReactNode;
};

export function DropdownItem({ value, disabled, className, onClick, submenu }: DropdownItemProps) {
  const itemRef = useRef(null);
  const submenuRef = useRef(null);
  const [isSubmenuOpen, setIsSubmenuOpen] = useState(false);

  return (
    <li
      ref={itemRef}
      className={`${styles['dropdown-item']} ${className && className} hover:bg-primary-100`}
      onClick={() => {
        !disabled && onClick && onClick(value);
      }}
      onMouseEnter={() => setIsSubmenuOpen(true)}
      onMouseLeave={() => setIsSubmenuOpen(false)}
    >
      {value}
      {submenu && isSubmenuOpen && <DropdownSubmenuContainer ref={submenuRef}>{submenu}</DropdownSubmenuContainer>}
    </li>
  );
}

type DropdownSubmenuContainerProps = {
  children: ReactNode;
};

export const DropdownSubmenuContainer = React.forwardRef<HTMLDivElement, DropdownSubmenuContainerProps>(({ children }, ref) => {
  return (
    <div ref={ref} className="z-10 absolute bg-light rounded shadow w-44 left-[-80%] -translate-y-7">
      <ul className="text-sm text-secondary-700">{children}</ul>
    </div>
  );
});

type DropdownProps = {
  title: string;
  options: Record<string, () => void>;
  align: 'left-0' | 'right-0';
  closeOnClick?: boolean;
  size?: 'xs' | 'sm' | 'md' | 'xl';
};

export function MenuDropdown({ title, options, align = 'left-0', closeOnClick = true, size = 'sm' }: DropdownProps) {
  const dropdownRef = useRef<HTMLDivElement>(null);
  const [isDropdownOpen, setIsDropdownOpen] = useState<boolean>(false);

  useEffect(() => {
    const handleClickOutside = (event: MouseEvent) => {
      if (dropdownRef.current && !dropdownRef.current.contains(event.target as Node)) {
        setIsDropdownOpen(false);
      }
    };
    if (isDropdownOpen) document.addEventListener('mousedown', handleClickOutside);
    return () => {
      document.removeEventListener('mousedown', handleClickOutside);
    };
  }, [isDropdownOpen]);

  return (
    <DropdownContainer ref={dropdownRef}>
      <DropdownButton title={title} onClick={(e) => setIsDropdownOpen(!isDropdownOpen)} size={size} />
      {isDropdownOpen && (
        <DropdownItemContainer align={align}>
          {options &&
            Object.keys(options).map((key, index) => (
              <DropdownItem
                key={index}
                value={key}
                onClick={() => {
                  options[key]();
                  closeOnClick && setIsDropdownOpen(false);
                }}
              />
            ))}
        </DropdownItemContainer>
      )}
    </DropdownContainer>
  );
}
