import { createSelector, createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import { globalConfigTypes, VisualizationConfiguration } from '../../vis/types';
import { isEqual } from 'lodash-es';
import { EncodingTypes } from '../../vis/configuration/encodings';
import { SettingTypes } from '../../vis/configuration/settings';
import { InteractionTypes } from '../../vis/configuration/interactions';

export type VisState = {
  activeVisualization?: string;
  settings: {
    general: globalConfigTypes;
  } & {
    [id: string]: VisualizationConfiguration;
  };
};

export const initialState: VisState = {
  activeVisualization: 'NodeLinkVis',
  settings: {
    general: {},
  },
};

export const visualizationSlice = createSlice({
  name: 'visualization',
  initialState,
  reducers: {
    addVisualization: (
      state,
      action: PayloadAction<{
        id: string;
        settings: SettingTypes;
        encodings: EncodingTypes;
        interactions: InteractionTypes;
      }>,
    ) => {
      const { id, settings, encodings, interactions } = action.payload;
      state.settings[id] = { settings, encodings, interactions };
    },
    removeVisualization: (state, action: PayloadAction<string>) => {
      if (state.settings[action.payload]) {
        delete state.settings[action.payload];
      }
    },
    setActiveVisualization: (state, action: PayloadAction<string>) => {
      state.activeVisualization = action.payload;
    },
    setVisualizationState: (state, action: PayloadAction<VisState>) => {
      if (action.payload.activeVisualization && !isEqual(action.payload, state)) {
        state.activeVisualization = action.payload.activeVisualization;
        state.settings = action.payload.settings;
      }
    },
    updateConfiguration: (state, action: PayloadAction<any>) => {
      const { general, visualization } = action.payload;
      state.settings.general = general;
      if (state.activeVisualization) state.settings[state.activeVisualization] = visualization;
    },
  },
});

export const { addVisualization, removeVisualization, setActiveVisualization, setVisualizationState, updateConfiguration } =
  visualizationSlice.actions;

export const visualizationState = (state: RootState) => state.visualize;
export const visualizationAllSettings = (state: RootState) => state.visualize.settings;

export default visualizationSlice.reducer;
