/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import React, { useState } from 'react';
import { Node, Handle, Position, NodeProps } from 'reactflow';
import styles from './relation.module.scss';
import { SchemaReactflowRelation, SchemaReactflowRelationWithFunctions } from '../../../model/reactflow';
import { QueryElementTypes } from '@graphpolaris/shared/lib/querybuilder';
import { Popup } from '@graphpolaris/shared/lib/components/Popup';
import { SchemaRelationshipPopup } from './SchemaRelationshipPopup';
import { SchemaEdge } from '../../../model';

/**
 * Relation node component that renders a relation node for the schema.
 * Can be dragged and dropped to the query builder.
 * @param {NodeProps} param0 The data of an entity flow element.
 */
export const RelationNode = React.memo(({ id, selected, data, ...props }: NodeProps<SchemaReactflowRelationWithFunctions>) => {
  const [openPopup, setOpenPopup] = useState(false);

  /**
   * Adds drag functionality in order to be able to drag the relationNode to the schema.
   * @param event React Mouse drag event.
   */
  const onDragStart = (event: React.DragEvent<HTMLDivElement>) => {
    const eventData: SchemaEdge = {
      type: QueryElementTypes.Relation,
      name: id, //TODO id?
      from: data.from,
      to: data.to,
      collection: data.collection,
      label: data.label,
      attributes: Array.from(data.attributes)
        .map((attribute) => attribute.attributes)
        .flat(),
    };
    event.dataTransfer.setData('application/reactflow', JSON.stringify(eventData));
    event.dataTransfer.effectAllowed = 'move';
  };

  /**
   * Displays the NodeQualityPopup when clicked, and hides them when clicked again
   */
  const onClickToggleNodeQualityPopup = (): void => {
    data.toggleNodeQualityPopup(data.collection);
  };

  /**
   * displays the attribute analytics popup menu when clicked, and hides them when clicked again
   */
  const onClickToggleAttributeAnalyticsPopupMenu = (): void => {
    data.toggleAttributeAnalyticsPopupMenu(data.collection);
  };

  return (
    <>
      {openPopup && (
        <Popup open={openPopup} hAnchor="left" className="-top-8" offset="-20rem">
          <SchemaRelationshipPopup data={data} onClose={() => setOpenPopup(false)} />
        </Popup>
      )}
      <div
        onDragStart={(event) => onDragStart(event)}
        onDragStartCapture={(event) => onDragStart(event)}
        onMouseDownCapture={(event) => {
          if (!event.shiftKey) event.stopPropagation();
        }}
        onClickCapture={(event) => {
          setOpenPopup(!openPopup);
        }}
        draggable
      >
        <div
          className={`rounded-sm transition-all duration-150 shadow shadow-dark/10 hover:shadow-md hover:shadow-dark/20 min-w-[4rem] max-w-[8rem] text-[0.8rem] bg-gradient-to-r pt-1 from-[#4893D4] to-[#1A476E]`}
        >
          <div className={`py-1 ${selected ? 'bg-secondary-300' : 'bg-secondary-100'}`}>
            <Handle
              style={{ pointerEvents: 'none' }}
              className={styles.handleTriangleTop}
              id="entitySourceLeft"
              position={Position.Top}
              type="target"
            ></Handle>
            <div className="p-2 py-1 text-center truncate">
              <span className="">{data.collection}</span>
            </div>

            <Handle
              className={styles.handleTriangleBottom}
              style={{ pointerEvents: 'none' }}
              position={Position.Bottom}
              type="source"
            ></Handle>
          </div>
        </div>
      </div>
    </>
  );
});

RelationNode.displayName = 'RelationNode';
