import React, { useEffect, useState } from 'react';
import { useAppDispatch } from '@graphpolaris/shared/lib/data-access';
import { addVisualization } from '../data-access/store/visualizationSlice';
import { useGraphQueryResult, useML, useSchemaGraph, useVisualization } from '@graphpolaris/shared/lib/data-access/store/hooks';
import { VISComponentType, globalConfigPropTypes } from './types';

export const Visualizations: Record<string, Function> = {
  TableVis: () => import('./visualizations/tablevis/tablevis'),
  PaohVis: () => import('./visualizations/paohvis/paohvis'),
  RawJSONVis: () => import('./visualizations/rawjsonvis/rawjsonvis'),
  NodeLinkVis: () => import('./visualizations/nodelinkvis/nodelinkvis'),
  // MapVis: () => import('./visualizations/mapvis/mapvis'),
  MatrixVis: () => import('./visualizations/matrixvis/matrixvis'),
};

export const VisualizationManager = () => {
  const dispatch = useAppDispatch();
  const vis = useVisualization();
  const graphQueryResult = useGraphQueryResult();
  const schema = useSchemaGraph();
  const ml = useML();

  const [visualizationComponent, setVisualizationComponent] = useState<VISComponentType>();

  useEffect(() => {
    if (vis.activeVisualization && vis.activeVisualization in Visualizations) {
      Visualizations[vis.activeVisualization]().then((r: any) => {
        setVisualizationComponent(r.default);
      });
    }
  }, [vis.activeVisualization]);

  useEffect(() => {
    if (visualizationComponent) {
      const { displayName, settings = {}, encodings = {}, interactions = {} } = visualizationComponent;
      dispatch(addVisualization({ id: displayName, settings, encodings, interactions }));
    }
  }, [visualizationComponent]);

  if (!visualizationComponent) {
    return <></>;
  }

  const globalConfig: globalConfigPropTypes = vis.settings.general
    ? Object.keys(vis.settings.general).reduce((propsObject, val) => {
        return {
          ...propsObject,
          [val]: vis.settings.general[val].value,
        };
      }, {})
    : {};

  let visSettings = {};
  let visEncodings = {};
  let visInteractions = {};

  if (vis.activeVisualization && vis.settings[vis.activeVisualization]) {
    const activeVisSettings = vis.settings[vis.activeVisualization]?.settings;
    const activeVisEncodings = vis.settings[vis.activeVisualization]?.encodings;
    const activeVisInteractions = vis.settings[vis.activeVisualization]?.interactions;

    visSettings = Object.keys(activeVisSettings ?? {}).reduce((propsObject, val) => {
      return {
        ...propsObject,
        [val]: activeVisSettings?.[val]?.value,
      };
    }, {});

    visEncodings = Object.keys(activeVisEncodings ?? {}).reduce((propsObject, val) => {
      return {
        ...propsObject,
        [val]: activeVisEncodings?.[val]?.marking,
      };
    }, {});

    visInteractions = Object.keys(activeVisInteractions ?? {}).reduce((propsObject, val) => {
      return {
        ...propsObject,
        [val]: activeVisInteractions?.[val]?.value,
      };
    }, {});
  }

  try {
    return (
      vis.activeVisualization && (
        <div className="w-full h-full">
          <visualizationComponent.VIS
            data={graphQueryResult}
            schema={schema}
            ml={ml}
            dispatch={dispatch}
            globalConfig={globalConfig}
            settings={visSettings}
            encodings={visEncodings}
            interactions={visInteractions}
          />
        </div>
      )
    );
  } catch (error) {
    return <div className="w-full h-full flex items-center justify-center">Something went wrong in the visualization component.</div>;
  }
};
