import React, { useState, useRef, useEffect } from 'react';
import { useAppDispatch, useGraphQueryResult, useQuerybuilderGraph, useVisualization } from '@graphpolaris/shared/lib/data-access';
import { LoadingSpinner } from '@graphpolaris/shared/lib/components/LoadingSpinner';
import { setActiveVisualization } from '@graphpolaris/shared/lib/data-access/store/visualizationSlice';
import { DropdownItem, DropdownItemContainer } from '@graphpolaris/shared/lib/components/dropdowns';
import ControlContainer from '@graphpolaris/shared/lib/components/controls';
import { Button } from '@graphpolaris/shared/lib/components/buttons';

import { VisualizationDialog } from './configuration';
import { Settings as SettingsIcon, Apps as AppsIcon, Fullscreen } from '@mui/icons-material';
import { VisualizationManager, Visualizations } from './visualizationManager';
import { Tooltip, TooltipContent, TooltipProvider, TooltipTrigger } from '../components/tooltip';

export const VisualizationPanel = () => {
  const graphQueryResult = useGraphQueryResult();
  const query = useQuerybuilderGraph();
  const dispatch = useAppDispatch();
  const vis = useVisualization();
  const [visDropdownOpen, setVisDropdownOpen] = useState<boolean>(false);
  const [showVisSettings, setShowVisSettings] = useState<boolean>(false);
  const visDropdownRef = useRef<HTMLDivElement>(null);

  useEffect(() => {
    const handleClickOutside = (event: MouseEvent) => {
      if (visDropdownRef.current && !visDropdownRef.current.contains(event.target as Node)) {
        setVisDropdownOpen(false);
      }
    };
    if (visDropdownOpen) document.addEventListener('mousedown', handleClickOutside);
    return () => {
      document.removeEventListener('mousedown', handleClickOutside);
    };
  }, [visDropdownOpen]);

  return (
    <div className="vis-panel h-full w-full overflow-y-auto" style={graphQueryResult.nodes.length === 0 ? { overflow: 'hidden' } : {}}>
      <VisualizationDialog open={showVisSettings} onClose={() => setShowVisSettings(false)} />
      <div className="sticky top-0 flex items-center justify-between z-[2] py-0 px-2 bg-secondary-100  border-b border-secondary-200">
        <h1 className="text-xs font-semibold text-secondary-800">{vis.activeVisualization} visualization</h1>
        <ControlContainer>
          <TooltipProvider delayDuration={0}>
            <Tooltip disabled={showVisSettings}>
              <TooltipTrigger>
                <Button
                  type="secondary"
                  variant="ghost"
                  size="xs"
                  iconComponent={<SettingsIcon />}
                  onClick={() => {
                    // TODO
                    // setShowVisSettings(!showVisSettings);
                  }}
                />
              </TooltipTrigger>
              <TooltipContent side={'bottom'} disabled={showVisSettings}>
                <p>Visualization settings</p>
              </TooltipContent>
            </Tooltip>
            <Tooltip>
              <TooltipTrigger>
                <Button
                  type="secondary"
                  variant="ghost"
                  size="xs"
                  iconComponent={<AppsIcon />}
                  onClick={() => {
                    setVisDropdownOpen(!visDropdownOpen);
                  }}
                />
              </TooltipTrigger>
              <TooltipContent side={'bottom'} disabled={visDropdownOpen}>
                <p>Change visualization</p>
              </TooltipContent>
            </Tooltip>
          </TooltipProvider>

          {visDropdownOpen && (
            <div ref={visDropdownRef}>
              <DropdownItemContainer align="top-6 right-6">
                {Object.keys(Visualizations).map((key) => (
                  <DropdownItem
                    key={key}
                    value={key}
                    onClick={() => {
                      setVisDropdownOpen(false);
                      dispatch(setActiveVisualization(key));
                    }}
                  />
                ))}
              </DropdownItemContainer>
            </div>
          )}
        </ControlContainer>
      </div>

      {graphQueryResult.queryingBackend ? (
        <div className="w-full h-full flex flex-col items-center justify-center overflow-hidden">
          <LoadingSpinner>Querying backend...</LoadingSpinner>
        </div>
      ) : graphQueryResult.nodes.length === 0 ? (
        <div className="w-full h-full flex flex-col items-center justify-center">
          <p>No data available to be shown</p>
          {query.nodes.length > 0 ? <p>Query resulted in empty dataset</p> : <p>Query for data to visualize</p>}
        </div>
      ) : (
        <div className="w-full h-full">
          <VisualizationManager />
        </div>
      )}
    </div>
  );
};
